@@ -1,8 +1,8 @@
 /* configure.c - core analysis suite
  *
  * Copyright (C) 1999, 2000, 2001, 2002 Mission Critical Linux, Inc.
- * Copyright (C) 2002, 2003, 2004, 2005, 2006, 2007, 2008 David Anderson
- * Copyright (C) 2002, 2003, 2004, 2005, 2006, 2007, 2008 Red Hat, Inc. All rights reserved.
+ * Copyright (C) 2002, 2003, 2004, 2005, 2006, 2007, 2008, 2009 David Anderson
+ * Copyright (C) 2002, 2003, 2004, 2005, 2006, 2007, 2008, 2009 Red Hat, Inc. All rights reserved.
  *
  * This program is free software; you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
@@ -941,7 +941,7 @@ strip_linefeeds(char *line)
         p = &LASTCHAR(line);
 
         while (*p == '\n')
-                *p = (char)NULL;
+                *p = '\0';
 
         return(line);
 }
@@ -1187,12 +1187,25 @@ make_spec_file(void)
 	printf("netdump, diskdump and kdump packages from Red Hat Linux, the mcore kernel patch\n");
 	printf("offered by Mission Critical Linux, or the LKCD kernel patch.\n");
 	printf("\n");
+	printf("%%package extensions\n");
+	printf("Summary: Additional commands for the crash dump analysis tool\n");
+	printf("Group: Development/Debuggers\n");
+	printf("\n");
+	printf("%%description extensions\n");
+	printf("The extensions package contains plugins that provide additional crash\n");
+	printf("commands. The extensions can be loaded in crash via the \"extend\" command.\n");
+	printf("\n");
+	printf("The following extensions are provided:\n");
+	printf("* sial:   Provides C-like language for writing dump analysis scripts\n");
+	printf("* dminfo: Device-mapper target analyzer\n");
+	printf("\n");
 	printf("%%prep\n");
         printf("%%setup -n %%{name}-%%{version}-%%{release}\n"); 
 	printf("# %%patch0 -p1 -b .install (patch example)\n");
 	printf("\n");
 	printf("%%build\n");
 	printf("make RPMPKG=\"%%{version}-%%{release}\"\n");
+	printf("make RPMPKG=\"%%{version}-%%{release}\" extensions\n");
      /*	printf("make crashd\n"); */
 	printf("\n");
 	printf("%%install\n");
@@ -1203,6 +1216,9 @@ make_spec_file(void)
 	printf("cp crash.8 %%{buildroot}%%{_mandir}/man8/crash.8\n");
 	printf("mkdir -p %%{buildroot}%%{_includedir}/crash\n");
 	printf("cp defs.h %%{buildroot}%%{_includedir}/crash\n");
+	printf("mkdir -p %%{buildroot}%%{_libdir}/crash/extensions\n");
+	printf("cp extensions/sial.so %%{buildroot}%%{_libdir}/crash/extensions\n");
+	printf("cp extensions/dminfo.so %%{buildroot}%%{_libdir}/crash/extensions\n");
 	printf("\n");
 	printf("%%clean\n");
 	printf("rm -rf %%{buildroot}\n");
@@ -1216,6 +1232,10 @@ make_spec_file(void)
 	printf("%%files devel\n");
 	printf("%%defattr(-,root,root)\n");
 	printf("%%{_includedir}/*\n");
+	printf("\n");
+	printf("%%files extensions\n");
+	printf("%%defattr(-,root,root)\n");
+	printf("%%{_libdir}/crash/extensions/*\n");
 }
 
 /*
