@@ -25,6 +25,7 @@ static int is_external_command(void);
 static int is_builtin_command(void);
 static int is_input_file(void);
 static void check_xen_hyper(void);
+static void show_untrusted_files(void);
 
 static struct option long_options[] = {
         {"memory_module", required_argument, 0, 0},
@@ -64,7 +65,7 @@ static struct option long_options[] = {
 int
 main(int argc, char **argv)
 {
-	int c, option_index;
+	int i, c, option_index;
 
 	setup_environment(argc, argv);
 
@@ -127,8 +128,17 @@ main(int argc, char **argv)
 				kt->flags |= SMP;
 
 		        else if (STREQ(long_options[option_index].name, 
-			    "machdep")) 
-				machdep->cmdline_arg = optarg;
+			    "machdep")) {
+				for (i = 0; i < MAX_MACHDEP_ARGS; i++) {
+					if (machdep->cmdline_args[i])
+						continue;
+					machdep->cmdline_args[i] = optarg;
+					break;
+				}
+				if (i == MAX_MACHDEP_ARGS)
+					error(INFO, "option ignored: %s\n",
+						optarg);
+			}
 
 		        else if (STREQ(long_options[option_index].name, 
 			    "version")) { 
@@ -296,7 +306,15 @@ main(int argc, char **argv)
 			break;
 
 		case 'm':
-			machdep->cmdline_arg = optarg;
+			for (i = 0; i < MAX_MACHDEP_ARGS; i++) {
+				if (machdep->cmdline_args[i])
+					continue;
+				machdep->cmdline_args[i] = optarg;
+				break;
+			}
+			if (i == MAX_MACHDEP_ARGS)
+				error(INFO, "option ignored: %s\n",
+					optarg);
 			break;
 
 		default:
@@ -395,7 +413,8 @@ main(int argc, char **argv)
                                 pc->writemem = write_xendump;
 
 			} else if (is_diskdump(argv[optind])) {
-                                if (pc->flags & MEMORY_SOURCES) {
+                                if ((pc->flags & MEMORY_SOURCES) &&
+                                    (!dumpfile_is_split())) {
                                         error(INFO,
                                             "too many dumpfile arguments\n");
                                         program_usage(SHORT_FORM);
@@ -493,6 +512,7 @@ main_loop(void)
 {
         if (!(pc->flags & GDB_INIT)) {
 		gdb_session_init();
+		show_untrusted_files();
 		if (XEN_HYPER_MODE()) {
 #ifdef XEN_HYPERVISOR_ARCH
 			machdep_init(POST_GDB);
@@ -876,6 +896,8 @@ setup_environment(int argc, char **argv)
 	                if ((afp = fopen(homerc, "r")) == NULL)
 	                        error(INFO, "cannot open %s: %s\n",
 	                                homerc, strerror(errno));
+			else if (untrusted_file(afp, homerc))
+				fclose(afp);
 	                else {
 	                        while (fgets(buf, BUFSIZE, afp))
 	                                resolve_rc_cmd(buf, ALIAS_RCHOME);
@@ -890,6 +912,8 @@ setup_environment(int argc, char **argv)
 		if ((afp = fopen(localrc, "r")) == NULL)
                         error(INFO, "cannot open %s: %s\n",
 				localrc, strerror(errno));
+		else if (untrusted_file(afp, localrc))
+			fclose(afp);
 		else {
 			while (fgets(buf, BUFSIZE, afp)) 
 				resolve_rc_cmd(buf, ALIAS_RCLOCAL);
@@ -1086,6 +1110,9 @@ dump_program_context(void)
         if (pc->flags & MINIMAL_MODE)
                 sprintf(&buf[strlen(buf)],
                         "%sMINIMAL_MODE", others++ ? "|" : "");
+        if (pc->flags & CRASHBUILTIN)
+                sprintf(&buf[strlen(buf)], 
+			"%sCRASHBUILTIN", others++ ? "|" : "");
 
 	if (pc->flags)
 		strcat(buf, ")");
@@ -1406,3 +1433,46 @@ check_xen_hyper(void)
 	error(FATAL, XEN_HYPERVISOR_NOT_SUPPORTED);
 #endif
 }
+
+/*
+ *  Reject untrusted .crashrc, $HOME/.crashrc, 
+ *  .gdbinit, and $HOME/.gdbinit files.
+ */
+static char *untrusted_file_list[4] = { 0 };
+
+int
+untrusted_file(FILE *filep, char *filename)
+{
+	struct stat sbuf;
+	int i;
+
+	if (filep && (fstat(fileno(filep), &sbuf) == 0) &&
+	    (sbuf.st_uid == getuid()) && !(sbuf.st_mode & S_IWOTH))
+		return FALSE;
+	
+	for (i = 0; i < 4; i++) {
+		if (!untrusted_file_list[i]) {
+			untrusted_file_list[i] = strdup(filename);
+			break;
+		}
+	}
+
+	return TRUE;
+}
+
+static void
+show_untrusted_files(void)
+{
+	int i, cnt;
+
+	for (i = cnt = 0; i < 4; i++) {
+		if (untrusted_file_list[i]) {
+			error(WARNING, "not using untrusted file: \"%s\"\n", 
+				untrusted_file_list[i]);
+			free(untrusted_file_list[i]);
+			cnt++;
+		}
+	}
+	if (cnt)
+		fprintf(fp, "\n");
+}
