@@ -36,7 +36,14 @@ static void check_dumpfile_size(char *);
 #define ELFREAD  0
 
 #define MIN_PAGE_SIZE (4096)
-	
+
+/* 
+ * PPC64 and IA64 architectures have configurable page sizes,
+ * which can differ from the host machine's page size.
+ */
+#define READ_PAGESIZE_FROM_VMCOREINFO() \
+	(machine_type("IA64") || machine_type("PPC64"))
+
 /*
  *  Determine whether a file is a netdump/diskdump/kdump creation, 
  *  and if TRUE, initialize the vmcore_data structure.
@@ -876,6 +883,10 @@ netdump_memory_dump(FILE *fp)
 			nd->xen_kdump_data->p2m_frames);
 		netdump_print("           xen_phys_start: %lx\n", 
 			nd->xen_kdump_data->xen_phys_start);
+		netdump_print("        xen_major_version: %d\n", 
+			nd->xen_kdump_data->xen_major_version);
+		netdump_print("        xen_minor_version: %d\n", 
+			nd->xen_kdump_data->xen_minor_version);
 		netdump_print("       p2m_mfn_frame_list: %lx\n", 
 			nd->xen_kdump_data->p2m_mfn_frame_list);
 		for (i = 0; i < nd->xen_kdump_data->p2m_frames; i++)
@@ -884,6 +895,8 @@ netdump_memory_dump(FILE *fp)
 		if (i) netdump_print("\n");
 	}
 	netdump_print("       num_prstatus_notes: %d\n", nd->num_prstatus_notes);	
+	netdump_print("               vmcoreinfo: %lx\n", (ulong)nd->vmcoreinfo);
+	netdump_print("          size_vmcoreinfo: %d\n", nd->size_vmcoreinfo);
 	netdump_print("       nt_prstatus_percpu: ");
         wrap = sizeof(void *) == SIZEOF_32BIT ? 8 : 4;
         flen = sizeof(void *) == SIZEOF_32BIT ? 8 : 16;
@@ -1387,6 +1400,96 @@ dump_Elf64_Phdr(Elf64_Phdr *prog, int store_pt_load_data)
 	netdump_print("                p_align: %lld\n", prog->p_align);
 }
 
+/*
+ * VMCOREINFO
+ *
+ * This is a ELF note intented for makedumpfile that is exported by the
+ * kernel that crashes and presented as ELF note to the /proc/vmcore
+ * of the panic kernel.
+ */
+
+#define VMCOREINFO_NOTE_NAME        "VMCOREINFO"
+#define VMCOREINFO_NOTE_NAME_BYTES  (sizeof(VMCOREINFO_NOTE_NAME))
+
+/*
+ * Reads a string value from VMCOREINFO.
+ *
+ * Returns a string (that has to be freed by the caller) that contains the
+ * value for key or NULL if the key has not been found.
+ */
+static char *
+vmcoreinfo_read_string(const char *key)
+{
+	int i, j, end;
+	size_t value_length;
+	size_t key_length = strlen(key);
+	char *vmcoreinfo = (char *)nd->vmcoreinfo;
+	char *value = NULL;
+
+	if (!nd->vmcoreinfo)
+		return NULL;
+
+	/* the '+ 1' is the equal sign */
+	for (i = 0; i < (nd->size_vmcoreinfo - key_length + 1); i++) {
+		/*
+		 * We must also check if we're at the beginning of VMCOREINFO
+		 * or the separating newline is there, and of course if we 
+		 * have a equal sign after the key.
+		 */
+		if ((strncmp(vmcoreinfo+i, key, key_length) == 0) &&
+		    (i == 0 || vmcoreinfo[i-1] == '\n') &&
+		    (vmcoreinfo[i+key_length] == '=')) {
+
+			end = -1;
+
+			/* Found -- search for the next newline. */
+			for (j = i + key_length + 1; 
+			     j < nd->size_vmcoreinfo; j++) {
+				if (vmcoreinfo[j] == '\n') {
+					end = j;
+					break;
+				}
+			}
+
+			/* 
+			 * If we didn't find an end, we assume it's the end 
+			 * of VMCOREINFO data. 
+			 */
+			if (end == -1) {
+				/* Point after the end. */
+				end = nd->size_vmcoreinfo + 1;
+			}
+
+			value_length = end - (1+ i + key_length);
+			value = malloc(value_length);
+			if (value)
+				strncpy(value, vmcoreinfo + i + key_length + 1, 
+					value_length);
+			break;
+		}
+	}
+
+	return value;
+}
+
+/*
+ * Reads an integer value from VMCOREINFO.
+ */
+static long
+vmcoreinfo_read_integer(const char *key, long default_value)
+{
+	char *string;
+	long retval = default_value;
+
+	string = vmcoreinfo_read_string(key);
+	if (string) {
+		retval = atol(string);
+		free(string);
+	}
+
+	return retval;
+}
+
 /*
  *  Dump a note section header -- the actual data is defined by netdump
  */
@@ -1525,6 +1628,8 @@ dump_Elf32_Nhdr(Elf32_Off offset, int store)
 					nd->xen_kdump_data->p2m_mfn = *(uptr+(words-1));
 				if (words > 9 && !nd->xen_kdump_data->xen_phys_start)
 					nd->xen_kdump_data->xen_phys_start = *(uptr+(words-2));
+				nd->xen_kdump_data->xen_major_version = *uptr;
+				nd->xen_kdump_data->xen_minor_version = *(uptr+1);
 			}
 		}
 		break;
@@ -1680,9 +1785,18 @@ dump_Elf64_Nhdr(Elf64_Off offset, int store)
 			if (store)
                         	error(WARNING, 
 				    "unknown Xen n_type: %lx\n\n", note->n_type);
-		} else if (vmcoreinfo)
+		} else if (vmcoreinfo) {
                         netdump_print("(unused)\n");
-                else 
+
+			if (READ_PAGESIZE_FROM_VMCOREINFO() && store) {
+				nd->vmcoreinfo = (char *)nd->elf64 + offset +
+					(sizeof(Elf64_Nhdr) +
+					((note->n_namesz + 3) & ~3));
+				nd->size_vmcoreinfo = note->n_descsz;
+				nd->page_size = (uint)
+					vmcoreinfo_read_integer("PAGESIZE", 0);
+			}
+                } else
                         netdump_print("(?)\n");
                 break;
 
@@ -1730,6 +1844,8 @@ dump_Elf64_Nhdr(Elf64_Off offset, int store)
 	                        	nd->xen_kdump_data->p2m_mfn = *(up+(words-1));
 				if (words > 9 && !nd->xen_kdump_data->xen_phys_start)
 					nd->xen_kdump_data->xen_phys_start = *(up+(words-2));
+				nd->xen_kdump_data->xen_major_version = *up;
+				nd->xen_kdump_data->xen_minor_version = *(up+1);
 			}
 		}
                 break;
@@ -1915,12 +2031,13 @@ get_netdump_regs_x86_64(struct bt_info *bt, ulong *ripp, ulong *rspp)
 void
 get_netdump_regs_x86(struct bt_info *bt, ulong *eip, ulong *esp)
 {
-	int i, search, panic, panic_task;
+	int i, search, panic, panic_task, altered;
 	char *sym;
 	ulong *up;
 	ulong ipintr_eip, ipintr_esp, ipintr_func;
 	ulong halt_eip, halt_esp;
 	int check_hardirq, check_softirq;
+	ulong stackbase, stacktop;
 
 	if (!is_task_active(bt->task)) {
 		machdep->get_stack_frame(bt, eip, esp);
@@ -1929,11 +2046,13 @@ get_netdump_regs_x86(struct bt_info *bt, ulong *eip, ulong *esp)
 
 	panic_task = tt->panic_task == bt->task ? TRUE : FALSE;
 
-	ipintr_eip = ipintr_esp = ipintr_func = panic = 0;
+	ipintr_eip = ipintr_esp = ipintr_func = panic = altered = 0;
 	halt_eip = halt_esp = 0;
 	check_hardirq = check_softirq = tt->flags & IRQSTACKS ? TRUE : FALSE;
 	search = ((bt->flags & BT_TEXT_SYMBOLS) && (tt->flags & TASK_INIT_DONE))
 		|| (machdep->flags & OMIT_FRAME_PTR);
+	stackbase = bt->stackbase;
+	stacktop = bt->stacktop;
 retry:
 	for (i = 0, up = (ulong *)bt->stackbuf; i < LONGS_PER_STACK; i++, up++){
 		sym = closest_symbol(*up);
@@ -2063,6 +2182,7 @@ next_sysrq:
 		alter_stackbuf(bt);
 		bt->flags |= BT_HARDIRQ;
 		check_hardirq = FALSE;
+		altered = TRUE;
 		goto retry;
 	}
 
@@ -2073,12 +2193,25 @@ next_sysrq:
 		alter_stackbuf(bt);
 		bt->flags |= BT_SOFTIRQ;
                 check_softirq = FALSE;
+		altered = TRUE;
                 goto retry;
         }
 
 	if (CRASHDEBUG(1))
 		error(INFO, 
     "get_netdump_regs_x86: cannot find anything useful (task: %lx)\n", bt->task);
+
+	if (altered) {
+                bt->stackbase = stackbase;
+                bt->stacktop = stacktop;
+		alter_stackbuf(bt);
+	}
+
+        if (XEN_CORE_DUMPFILE() && !panic_task && is_task_active(bt->task) &&
+	    !(bt->flags & (BT_TEXT_SYMBOLS_ALL|BT_TEXT_SYMBOLS)))
+		error(FATAL, 
+		    "starting backtrace locations of the active (non-crashing) "
+		    "xen tasks\n    cannot be determined: try -t or -T options\n");
  
 	machdep->get_stack_frame(bt, eip, esp);
 }
@@ -2343,3 +2476,15 @@ xen_phys_start(void)
 {
 	return nd->xen_kdump_data->xen_phys_start;
 }
+
+int
+xen_major_version(void)
+{
+	return nd->xen_kdump_data->xen_major_version;
+}
+
+int
+xen_minor_version(void)
+{
+	return nd->xen_kdump_data->xen_minor_version;
+}
